@@ -50,11 +50,11 @@ class QtAbstractButton(QtControl, ProxyAbstractButton):
         super(QtAbstractButton, self).init_widget()
         d = self.declaration
         if d.text:
-            self.set_text(d.text, guard=False)
+            self.set_text(d.text)
         if d.icon:
-            self.set_icon(d.icon, guard=False)
+            self.set_icon(d.icon)
         if -1 not in d.icon_size:
-            self.set_icon_size(d.icon_size, guard=False)
+            self.set_icon_size(d.icon_size)
         self.set_checkable(d.checkable)
         self.set_checked(d.checked)
         widget = self.widget
@@ -86,17 +86,14 @@ class QtAbstractButton(QtControl, ProxyAbstractButton):
     #--------------------------------------------------------------------------
     # ProxyAbstractButton API
     #--------------------------------------------------------------------------
-    def set_text(self, text, guard=True):
+    def set_text(self, text):
         """ Sets the widget's text with the provided value.
 
         """
-        if guard:
-            with self.geometry_guard():
-                self.widget.setText(text)
-        else:
+        with self.geometry_guard():
             self.widget.setText(text)
 
-    def set_icon(self, icon, guard=True):
+    def set_icon(self, icon):
         """ Set the icon on the widget.
 
         """
@@ -104,20 +101,14 @@ class QtAbstractButton(QtControl, ProxyAbstractButton):
             qicon = get_cached_qicon(icon)
         else:
             qicon = QIcon()
-        if guard:
-            with self.geometry_guard():
-                self.widget.setIcon(qicon)
-        else:
+        with self.geometry_guard():
             self.widget.setIcon(qicon)
 
-    def set_icon_size(self, size, guard=True):
+    def set_icon_size(self, size):
         """ Sets the widget's icon size.
 
         """
-        if guard:
-            with self.geometry_guard():
-                self.widget.setIconSize(QSize(*size))
-        else:
+        with self.geometry_guard():
             self.widget.setIconSize(QSize(*size))
 
     def set_checkable(self, checkable):
