@@ -84,10 +84,14 @@ class QtConstraintsWidget(QtWidget, ProxyConstraintsWidget):
     def geometry_guard(self):
         """ A context manager for guarding the geometry of the widget.
 
-        This manager will call 'geometry_updated' if the size hint,
-        minimum, or maximum size of the widget has changed.
+        If the proxy is fully active, this context manager will call the
+        'geometry_updated' method if the size hint, minimum, or maximum
+        size of the widget changes during context execution.
 
         """
+        if not self.is_active:
+            yield
+            return
         widget = self.widget
         old_hint = widget.sizeHint()
         old_min = widget.minimumSize()
@@ -97,3 +101,15 @@ class QtConstraintsWidget(QtWidget, ProxyConstraintsWidget):
             old_min != widget.minimumSize() or
             old_max != widget.maximumSize()):
             self.geometry_updated()
+
+    #--------------------------------------------------------------------------
+    # Reimplementations
+    #--------------------------------------------------------------------------
+    def set_font(self, font):
+        """ A reimplemented font setter.
+
+        This method sets the font from within a geometry guard.
+
+        """
+        with self.geometry_guard():
+            super(QtConstraintsWidget, self).set_font(font)
