@@ -65,7 +65,7 @@ class QtLayoutItem(LayoutItem):
 
         Returns
         -------
-        result : Contrainable
+        result : Constrainable
             An object which implements the Constrainable interface.
 
         """
