@@ -1,5 +1,5 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2013, Nucleic Development Team.
+# Copyright (c) 2014, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -7,14 +7,18 @@
 #------------------------------------------------------------------------------
 from atom.api import Typed, Coerced
 
+from enaml.drag_drop import DropAction
 from enaml.styling import StyleCache
 from enaml.widgets.widget import Feature, ProxyWidget
 
-from .QtCore import Qt, QSize
-from .QtGui import QFont, QWidget, QWidgetAction, QApplication
+from .QtCore import Qt, QSize, QPoint
+from .QtGui import QFont, QWidget, QWidgetAction, QApplication, QDrag, QPixmap
 
 from . import focus_registry
-from .q_resource_helpers import get_cached_qcolor, get_cached_qfont
+from .q_resource_helpers import (
+    get_cached_qcolor, get_cached_qfont, get_cached_qimage
+)
+from .qt_drag_drop import QtDropEvent
 from .qt_toolkit_object import QtToolkitObject
 from .styleutil import translate_style
 
@@ -34,6 +38,9 @@ class QtWidget(QtToolkitObject, ProxyWidget):
     #: Internal storage for the shared widget action.
     _widget_action = Typed(QWidgetAction)
 
+    #: Internal storage for the drag origin position.
+    _drag_origin = Typed(QPoint)
+
     #--------------------------------------------------------------------------
     # Initialization API
     #--------------------------------------------------------------------------
@@ -106,6 +113,10 @@ class QtWidget(QtToolkitObject, ProxyWidget):
             self.hook_focus_traversal()
         if features & Feature.FocusEvents:
             self.hook_focus_events()
+        if features & Feature.DragEnabled:
+            self.hook_drag()
+        if features & Feature.DropEnabled:
+            self.hook_drop()
 
     def _teardown_features(self):
         """ Teardowns the advanced widget feature handlers.
@@ -118,6 +129,10 @@ class QtWidget(QtToolkitObject, ProxyWidget):
             self.unhook_focus_traversal()
         if features & Feature.FocusEvents:
             self.unhook_focus_events()
+        if features & Feature.DragEnabled:
+            self.unhook_drag()
+        if features & Feature.DropEnabled:
+            self.unhook_drop()
 
     #--------------------------------------------------------------------------
     # Protected API
@@ -157,11 +172,13 @@ class QtWidget(QtToolkitObject, ProxyWidget):
 
         """
         widget = self.focus_target()
-        if ((widget.focusPolicy() & Qt.TabFocus) and
-            widget.isEnabled() and
-            widget.isVisibleTo(widget.window())):
-            widget.setFocus(reason)
-            return True
+        if not widget.focusPolicy & Qt.TabFocus:
+            return False
+        if not widget.isEnabled():
+            return False
+        if not widget.isVisibleTo(widget.window()):
+            return False
+        widget.setFocus(reason)
         return False
 
     def focus_target(self):
@@ -241,6 +258,121 @@ class QtWidget(QtToolkitObject, ProxyWidget):
         type(widget).focusOutEvent(widget, event)
         self.declaration.focus_lost()
 
+    def hook_drag(self):
+        """ Install the hooks for drag operations.
+
+        """
+        widget = self.widget
+        widget.mousePressEvent = self.mousePressEvent
+        widget.mouseMoveEvent = self.mouseMoveEvent
+        widget.mouseReleaseEvent = self.mouseReleaseEvent
+
+    def unhook_drag(self):
+        """ Remove the hooks for drag operations.
+
+        """
+        widget = self.widget
+        del widget.mousePressEvent
+        del widget.mouseMoveEvent
+        del widget.mouseReleaseEvent
+
+    def mousePressEvent(self, event):
+        """ Handle the mouse press event for a drag operation.
+
+        """
+        if event.button() == Qt.LeftButton:
+            self._drag_origin = event.pos()
+        widget = self.widget
+        type(widget).mousePressEvent(widget, event)
+
+    def mouseMoveEvent(self, event):
+        """ Handle the mouse move event for a drag operation.
+
+        """
+        if event.buttons() & Qt.LeftButton and self._drag_origin is not None:
+            dist = (event.pos() - self._drag_origin).manhattanLength()
+            if dist >= QApplication.startDragDistance():
+                self.do_drag()
+                self._drag_origin = None
+                return
+        widget = self.widget
+        type(widget).mouseMoveEvent(widget, event)
+
+    def mouseReleaseEvent(self, event):
+        """ Handle the mouse release event for the drag operation.
+
+        """
+        if event.button() == Qt.LeftButton:
+            self._drag_origin = None
+        widget = self.widget
+        type(widget).mouseReleaseEvent(widget, event)
+
+    def hook_drop(self):
+        """ Install hooks for drop operations.
+
+        """
+        widget = self.widget
+        widget.setAcceptDrops(True)
+        widget.dragEnterEvent = self.dragEnterEvent
+        widget.dragMoveEvent = self.dragMoveEvent
+        widget.dragLeaveEvent = self.dragLeaveEvent
+        widget.dropEvent = self.dropEvent
+
+    def unhook_drop(self):
+        """ Remove hooks for drop operations.
+
+        """
+        widget = self.widget
+        widget.setAcceptDrops(False)
+        del widget.dragEnterEvent
+        del widget.dragMoveEvent
+        del widget.dragLeaveEvent
+        del widget.dropEvent
+
+    def do_drag(self):
+        """ Perform the drag operation for the widget.
+
+        """
+        drag_data = self.declaration.drag_start()
+        if drag_data is None:
+            return
+        widget = self.widget
+        qdrag = QDrag(widget)
+        qdrag.setMimeData(drag_data.mime_data.q_data())
+        if drag_data.image is not None:
+            qimg = get_cached_qimage(drag_data.image)
+            qdrag.setPixmap(QPixmap.fromImage(qimg))
+        else:
+            qdrag.setPixmap(QPixmap.grabWidget(widget))
+        default = Qt.DropAction(drag_data.default_drop_action)
+        supported = Qt.DropActions(drag_data.supported_actions)
+        qresult = qdrag.exec_(supported, default)
+        self.declaration.drag_end(drag_data, DropAction(int(qresult)))
+
+    def dragEnterEvent(self, event):
+        """ Handle the drag enter event for the widget.
+
+        """
+        self.declaration.drag_enter(QtDropEvent(event))
+
+    def dragMoveEvent(self, event):
+        """ Handle the drag move event for the widget.
+
+        """
+        self.declaration.drag_move(QtDropEvent(event))
+
+    def dragLeaveEvent(self, event):
+        """ Handle the drag leave event for the widget.
+
+        """
+        self.declaration.drag_leave()
+
+    def dropEvent(self, event):
+        """ Handle the drop event for the widget.
+
+        """
+        self.declaration.drop(QtDropEvent(event))
+
     #--------------------------------------------------------------------------
     # Framework API
     #--------------------------------------------------------------------------
